function [p,j] = readlna(f,s,w,i)
% [P,IXO] = readlna(F,S,W,IX)  Read segment S of the lna file F of W elems
%     F is the filename of a standard Cambridge LNA file (byte
%     encoding of log probabilities).  Read this in the S'th segment 
%     and convert it to linear probabilities.  S counts from 1; S=0
%     (the default) means to read the entire file in at once.  
%     W specifies the number of values per frame (default 54).
%     IXO returns a vector of the number of frames in each segment, 
%     although only the segments up to the returned segment are guaranteed 
%     to be present.  However, if S<0, P is returned empty, and IXO 
%     contains the lengths of every segment in the file.   To speed up 
%     access to later segments by avoiding the rescanning of segment 
%     boundaries, the partial or complete segment length index for 
%     this file can be passed in as IX.  To read through an entire file, 
%     the best approach is: 
%     [d,ix] = readlna(f,-1,w); for n=1:size(ix,2); d = readlna(f,n,w,ix); ..
%     Note: ix can be specified as a 'fake' index to subdivide existing 
%     segments, but any read will segment will not extend beyond an
%     actual EOS mark.
%    
% 1998nov04 dpwe@icsi.berkeley.edu
% $Header: /n/yam/da/dpwe/matlab/icsifns/RCS/readlna.m,v 1.5 1999/04/18 01:25:19 dpwe Exp $

if nargin < 2
  s = 0;
end
if nargin < 3
  w = 54;
end
if nargin < 4
  segstarts = [0];
else
  segstarts = [0, cumsum(i)];
end

fid = fopen(f);

if (s == 0) 
  lna = fread(fid, [w+1,inf], 'uchar');
  lna = lna([2:(w+1)],:);
else
  % Read through a longer lna
  buflen = 10000; % frames per read

  desseglen = inf; % No desired seglen at present
  
  if s < 0
    s = inf;   % we're just returning seglens, so scan to end
  end
  
  % segstarts = [0];
  segnum = 1;
  frpos = 0;
  lna = [];
  
  % Maybe skip with existing segstarts array
  if size(segstarts,2) > 1
    segnum = min(s, size(segstarts,2));
    frpos = segstarts(segnum);
    if fseek(fid, frpos*(w+1), 'bof') < 0
      error('seeking to segment start');
    end
    % Were we provided with guidance on the length of the target segment
    if size(segstarts,2) > s
      desseglen = min(desseglen, segstarts(s+1) - segstarts(s));
    end
    % The length of the segstarts array is used to keep track of 
    % which segment we're currently reading - trim it appropriately
    segstarts = segstarts(1:segnum);
  end
  
  while feof(fid) == 0 & segnum <= s
    %disp(['read ',num2str(w+1),'x',num2str(buflen)])
    lnabuf = fread(fid, [w+1, buflen], 'uchar');
    nread = size(lnabuf,2);
    % disp(['read ', num2str(nread), ' at pos ', num2str(frpos)]);
    segbdys = (lnabuf(1,:) > 127);
    segstarts = [segstarts, frpos + find(segbdys)];
    segnum = size(segstarts,2);
    if segnum >= s
      % in the middle of our target segment - take it!
      segstart = segstarts(s);
      if (segnum > s)
        segend = segstarts(s+1);
      else
        segend = inf;
      end
      % apply any predefined desired seg len as a possible minimum
      segend = min(segend, desseglen+segstart);
      lna = [lna, lnabuf([2:(w+1)], [(1+max(0, segstart-frpos)):(min(nread, segend-frpos))])];
    end
    frpos = frpos + nread;
  end
end

fclose(fid);

% return seglens, as many as we have
nsegs = size(segstarts, 2) - 1;
seglens = segstarts(2:(nsegs+1)) - segstarts(1:nsegs);
j = seglens;

% do conversion of the LNA values
k = 0.00000;
p = exp(-(lna+.5)/24) - k;
